"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var TypeOrmAlsModule_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.TypeOrmAlsModule = void 0;
const common_1 = require("@nestjs/common");
const async_hooks_1 = require("async_hooks");
const typeorm_1 = require("typeorm");
const core_1 = require("@nestjs/core");
const typeorm_als_constants_1 = require("./typeorm-als.constants");
const typeorm_als_utils_1 = require("./typeorm-als.utils");
let TypeOrmAlsModule = TypeOrmAlsModule_1 = class TypeOrmAlsModule {
    constructor(asyncStorage, connection, discovery) {
        this.asyncStorage = asyncStorage;
        this.connection = connection;
        this.discovery = discovery;
    }
    static forRoot(options) {
        const asyncLocalStorage = this.createALS();
        const alsProvider = {
            provide: typeorm_als_constants_1.ASYNC_STORAGE,
            useValue: asyncLocalStorage,
        };
        const optionsProvider = {
            provide: typeorm_als_constants_1.TYPEORM_ALS_MODULE_OPTIONS,
            useValue: Object.assign(Object.assign({}, typeorm_als_constants_1.TYPEORM_ALS_MODULE_DEFAULT_OPTIONS), options),
        };
        return {
            module: TypeOrmAlsModule_1,
            imports: [core_1.DiscoveryModule],
            providers: [alsProvider, optionsProvider, core_1.DiscoveryService],
            exports: [alsProvider, optionsProvider],
        };
    }
    static forRootAsync(options) {
        const asyncLocalStorage = this.createALS();
        const alsProvider = {
            provide: typeorm_als_constants_1.ASYNC_STORAGE,
            useValue: asyncLocalStorage,
        };
        let optionsProvider;
        if (options) {
            optionsProvider = {
                provide: typeorm_als_constants_1.TYPEORM_ALS_MODULE_OPTIONS,
                useFactory: options.useFactory,
                inject: options.inject || [],
            };
        }
        else {
            optionsProvider = {
                provide: typeorm_als_constants_1.TYPEORM_ALS_MODULE_OPTIONS,
                useValue: typeorm_als_constants_1.TYPEORM_ALS_MODULE_DEFAULT_OPTIONS,
            };
        }
        return {
            module: TypeOrmAlsModule_1,
            imports: [core_1.DiscoveryModule],
            providers: [alsProvider, optionsProvider, core_1.DiscoveryService],
            exports: [alsProvider, optionsProvider],
        };
    }
    static createALS() {
        return new async_hooks_1.AsyncLocalStorage();
    }
    onModuleInit() {
        const wrappers = this.discovery.getProviders();
        wrappers.forEach((wrapper) => {
            const instance = wrapper.instance;
            const asyncStorage = this.asyncStorage;
            if (instance instanceof typeorm_1.Repository) {
                Object.assign(instance, { _manager: instance.manager });
                Object.defineProperty(instance, 'manager', {
                    get() {
                        const store = asyncStorage.getStore();
                        if (store) {
                            const entityManager = typeorm_als_utils_1.getEntityManager(store, this._manager.connection);
                            if (entityManager) {
                                return entityManager;
                            }
                        }
                        return this._manager;
                    },
                });
            }
            else if (instance instanceof typeorm_1.Connection) {
                Object.assign(instance, {
                    _createQueryBuilder: instance.createQueryBuilder,
                });
                Object.defineProperty(instance, 'createQueryBuilder', {
                    configurable: true,
                    value(entityOrRunner, alias, queryRunner) {
                        const store = asyncStorage.getStore();
                        let existingQueryRunner;
                        if (store) {
                            existingQueryRunner = typeorm_als_utils_1.getQueryRunner(store, instance);
                        }
                        if (queryRunner ||
                            (!alias && entityOrRunner) ||
                            !existingQueryRunner) {
                            return this._createQueryBuilder(entityOrRunner, alias, queryRunner);
                        }
                        if (!alias) {
                            return this._createQueryBuilder(existingQueryRunner);
                        }
                        else {
                            return this._createQueryBuilder(entityOrRunner, alias, existingQueryRunner);
                        }
                    },
                });
            }
        });
    }
    configure(consumer) {
        const middleware = (req, res, next) => {
            let store = this.asyncStorage.getStore();
            if (!store) {
                store = new Map();
                this.asyncStorage.run(store, () => {
                    next();
                });
            }
        };
        consumer.apply(middleware).forRoutes('*');
    }
};
TypeOrmAlsModule = TypeOrmAlsModule_1 = __decorate([
    common_1.Global(),
    common_1.Module({}),
    __param(0, common_1.Inject(typeorm_als_constants_1.ASYNC_STORAGE)),
    __metadata("design:paramtypes", [async_hooks_1.AsyncLocalStorage,
        typeorm_1.Connection,
        core_1.DiscoveryService])
], TypeOrmAlsModule);
exports.TypeOrmAlsModule = TypeOrmAlsModule;
